deferred class DIRECTION
inherit
	KNOWS_RAND
feature
	direction_idle : INTEGER is 0
	min_dir : INTEGER is 1
	north : INTEGER is 1
	south : INTEGER is 2
	west : INTEGER is 3
	east : INTEGER is 4
	max_dir : INTEGER is 4
	direction_flow : INTEGER is 5
	direction_opp : INTEGER is 6
	direction_cw : INTEGER is 7
	direction_ccw : INTEGER is 8
	direction_rndns : INTEGER is 9
	direction_rndne : INTEGER is 10
	direction_rndp : INTEGER is 11
	direction_seek : INTEGER is 12

	string_to_direction(s : STRING) : INTEGER is
	do
		if string_direction_table.has(s) then
			Result := string_direction_table.at(s)
		end
	end

	opposite_direction(d : INTEGER) : INTEGER is
	do
		inspect d
		when north then
			Result := south
		when south then
			Result := north
		when west then
			Result := east
		when east then
			Result := west
		else
			Result := d
		end
	end

	clockwise_direction(d : INTEGER) : INTEGER is
	do
		inspect d
		when north then
			Result := east
		when south then
			Result := west
		when west then
			Result := north
		when east then
			Result := south
		else
			Result := d
		end
	end

	counterclockwise_direction(d : INTEGER) : INTEGER is
	do
		inspect d
		when north then
			Result := west
		when south then
			Result := east
		when west then
			Result := south
		when east then
			Result := north
		else
			Result := d
		end
	end

	direction_string : ARRAY[STRING] is
	once
		!!Result.make(min_dir, max_dir)
		Result.put("north", north)
		Result.put("south", south)
		Result.put("east", east)
		Result.put("west", west)
	end

	string_direction_table : DICTIONARY[INTEGER, STRING] is
	once
		!!Result.make
		Result.put(north, "north")
		Result.put(north, "n")
		Result.put(south, "south")
		Result.put(south, "s")
		Result.put(east, "east")
		Result.put(east, "e")
		Result.put(west, "west")
		Result.put(west, "w")
		Result.put(direction_idle, "idle")
		Result.put(direction_flow, "flow")
		Result.put(direction_opp, "opp")
		Result.put(direction_cw, "cw")
		Result.put(direction_ccw, "ccw")
		Result.put(direction_rndns, "rndns")
		Result.put(direction_rndne, "rndne")
		Result.put(direction_rndp, "rndp")
		Result.put(direction_seek, "seek")
	end

	direction_from_xy(x, y : INTEGER) : INTEGER is
	do
		if x = 0 then
			if y = 1 then
				Result := south
			else
				Result := north
			end
		else
			if x = 1 then
				Result := east
			else
				Result := west
			end
		end
	end

	seek_direction(src : LOCATION; tgt : LOCATION) : INTEGER is
	local
		choicex, choicey : INTEGER
		x, y : INTEGER
	do
		x := tgt.x - src.x
		y := tgt.y - src.y
		if x = 0 then
			if y > 0 then
				Result := south
			else
				Result := north
			end
		elseif x > 0 then
			if y = 0 then
				Result := east
			else
				choicex := east
				if y > 0 then
					choicey := south
				else
					choicey := north
				end
				rand.next
				--if rand.last_integer(x + y) <= x then
				if rand.last_real < 0.5 then
					Result := choicex
				else
					Result := choicey
				end
			end
		else
			if y = 0 then
				Result := west
			else
				choicex := west
				if y > 0 then
					choicey := south
				else
					choicey := north
				end
				rand.next
				--if rand.last_integer(x + y) <= x then
				if rand.last_real < 0.5 then
					Result := choicex
				else
					Result := choicey
				end
			end
		end
	end
end
