#include <stdio.h>
#include "SDL.h"
#include "png.h"

struct info {
	Uint32 palette[16];
	Uint8   bpp;
} glob;
SDL_Surface *screen;

void ext_sdl_init(void) {
	SDL_Init(SDL_INIT_AUDIO|SDL_INIT_VIDEO|SDL_INIT_TIMER);
	atexit(SDL_Quit);

	screen = SDL_SetVideoMode(640, 480, 0, SDL_FULLSCREEN);
	glob.bpp = screen->format->BytesPerPixel;

	glob.palette[0] = SDL_MapRGB(screen->format, 0x00, 0x00, 0x00);
	glob.palette[1] = SDL_MapRGB(screen->format, 0x00, 0x00, 0xAF);
	glob.palette[2] = SDL_MapRGB(screen->format, 0x00, 0x9F, 0x00);
	glob.palette[3] = SDL_MapRGB(screen->format, 0x00, 0x9F, 0x9F);
	glob.palette[4] = SDL_MapRGB(screen->format, 0x7F, 0x00, 0x00);
	glob.palette[5] = SDL_MapRGB(screen->format, 0x7F, 0x00, 0x7F);
	glob.palette[6] = SDL_MapRGB(screen->format, 0x7F, 0x7F, 0x00);
	glob.palette[7] = SDL_MapRGB(screen->format, 0x9F, 0x9F, 0x9F);
	glob.palette[8] = SDL_MapRGB(screen->format, 0x5F, 0x5F, 0x5F);
	glob.palette[9] = SDL_MapRGB(screen->format, 0x7F, 0x7F, 0xFF);
	glob.palette[10] = SDL_MapRGB(screen->format, 0x7F, 0xFF, 0x7F);
	glob.palette[11] = SDL_MapRGB(screen->format, 0x7F, 0xFF, 0xFF);
	glob.palette[12] = SDL_MapRGB(screen->format, 0xFF, 0x5F, 0x5F);
	glob.palette[13] = SDL_MapRGB(screen->format, 0xFF, 0x5F, 0xFF);
	glob.palette[14] = SDL_MapRGB(screen->format, 0xFF, 0xFF, 0x7F);
	glob.palette[15] = SDL_MapRGB(screen->format, 0xFF, 0xFF, 0xFF);
}

void *ext_make_flag(void) {
	Uint32 *res;
	res = (Uint32 *) malloc(sizeof(Uint32));
	return(res);
}

#if 0
void ext_draw_pixel(Sint32 X, Sint32 Y, int colour) {
         Uint32   pixel;
         Uint8   *bits;

         /* Map the color yellow to this display (R=0xFF, G=0xFF, B=0x00)
            Note:  If the display is palettized, you must set the palette first.
         */


		 pixel = glob.palette[colour];
	//pixel = SDL_MapRGB(screen->format, 0x7F, 0x7F, 0x7F);
		bits = ((Uint8 *)screen->pixels)+Y*screen->pitch+X;

		 *((Uint8 *)(bits)) = (Uint8)pixel;

         return;
}
#endif

void ext_draw_pixel(Sint32 X, Sint32 Y, int colour) {
         Uint32   pixel;
         Uint8   *bits, bpp;

         /* Map the color yellow to this display (R=0xFF, G=0xFF, B=0x00)
            Note:  If the display is palettized, you must set the palette first.
         */


		 pixel = glob.palette[colour];
	//pixel = SDL_MapRGB(screen->format, 0x7F, 0x7F, 0x7F);
		 bpp = glob.bpp;
		bits = ((Uint8 *)screen->pixels)+Y*screen->pitch+X*bpp;

         /* Set the pixel */
         switch(bpp) {
                 case 1:
                         *((Uint8 *)(bits)) = (Uint8)pixel;
                         break;
                 case 2:
                         *((Uint16 *)(bits)) = (Uint16)pixel;
                         break;
                 case 3: { /* Format/endian independent */
                         Uint8 r, g, b;

                         r = (pixel>>screen->format->Rshift)&0xFF;
                         g = (pixel>>screen->format->Gshift)&0xFF;
                         b = (pixel>>screen->format->Bshift)&0xFF;
                         *((bits)+screen->format->Rshift/8) = r; 
                         *((bits)+screen->format->Gshift/8) = g;
                         *((bits)+screen->format->Bshift/8) = b;
                         }
                         break;
                 case 4:
                         *((Uint32 *)(bits)) = (Uint32)pixel;
                         break;
         }

         return;
}

void ext_start_drawing(void) {
         /* Calculate the framebuffer offset of the center of the screen */
         if ( SDL_MUSTLOCK(screen) ) {
                 if ( SDL_LockSurface(screen) < 0 )
                         return;
         }
		return;
}

void ext_finish_drawing(void) {
         /* Update the display */
         if ( SDL_MUSTLOCK(screen) ) {
                 SDL_UnlockSurface(screen);
         }
         SDL_UpdateRect(screen, 0, 0, 0, 0);

         return;
}

static Uint32 tick_count;
static Uint32 tick_interval = 50;

void ext_set_tick_interval(int t) {
	tick_interval = (Uint32) t;
}

SDL_Event key_event;
SDL_Event dummy_event;
int key_ready;

void ext_tick_reset(void) {
	tick_count = SDL_GetTicks();
	key_ready = 0;
}

void ext_flush_event(void) {
	while (SDL_PollEvent(&dummy_event));	//flush event queue
}

int ext_last_key(void) {
	ext_flush_event();
	key_ready = 0;
	return (int) key_event.key.keysym.sym;
}

const int repeatable[] = { SDLK_LEFT, SDLK_RIGHT, SDLK_UP, SDLK_DOWN };

int ext_key_ready(void) {
	Uint8 *keys;

	int i;

	if (key_ready) {
		return 1;
	} else {
		while (SDL_PollEvent(&key_event)) {
			if (key_event.type == SDL_KEYDOWN) {
				key_ready = 1;
				return 1;
			}
		}
		// Temporary hack until I can use SDL_EnableKeyRepeat
		if (!key_ready) {
			keys = SDL_GetKeyState(NULL);
			for (i=0; i<4; i++) {
				if ( keys[repeatable[i]] == SDL_PRESSED ) {
					key_ready = 1;
					key_event.key.keysym.sym = repeatable[i];
					return 1;
				}
			}
			/*
			if ( keys[SDLK_LEFT] == SDL_PRESSED ) {
				key_ready = 1;
				key_event.key.keysym.sym = SDLK_LEFT;
				return 1;
			}
			if ( keys[SDLK_RIGHT] == SDL_PRESSED ) {
				key_ready = 1;
				key_event.key.keysym.sym = SDLK_RIGHT;
				return 1;
			}
			*/
		}
	}
	return 0;
}

int ext_tick_ready(void) {

	if (tick_interval + tick_count < SDL_GetTicks()) {
		//tick_count = SDL_GetTicks();
		tick_count += tick_interval;
		//printf("%d\n", tick_count);
		return 1;
	}
	if (!key_ready) {
		while (SDL_PollEvent(&key_event)) {
			if (key_event.type == SDL_KEYDOWN) {
				break;
			}
		}
	}
	return 0;
}

int ext_get_key(void) {
	for(;;) {
		if (SDL_PollEvent(&key_event)) {
			if (key_event.type == SDL_KEYDOWN) {
				key_ready = 0;
				break;
			}
		}
	}

	ext_flush_event();
	return (int) key_event.key.keysym.sym;
}

int ext_tile_width = 32, ext_tile_height = 32;
void ext_set_tile_size(int w, int h) {
	ext_tile_width = w;
	ext_tile_height = h;
}

void *ext_read_png(char *filename) {
	char header[8];
	int is_png;
	png_structp png_ptr;
	png_infop info_ptr, end_info;
	png_uint_32 width, height;
	png_uint_32 row;
	png_bytep* row_pointers;
	int i, j;
	FILE *fp;

	char *res;

	res = (char *) malloc(sizeof(char) * ext_tile_width * ext_tile_height * 4);

    fp = fopen(filename, "rb");
    if (!fp)
    {
        return (void *) res;
    }

    fread(header, 1, 8, fp);
    is_png = !png_sig_cmp(header, 0, 8);
	
    if (!is_png)
    {
        return (void *) res;
    }

	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);

	if (png_ptr == NULL)
	{
		fclose(fp);
        return (void *) res;
	}

	info_ptr = png_create_info_struct(png_ptr);
	if (info_ptr == NULL)
	{
		fclose(fp);
		png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
		return;
	}

    end_info = png_create_info_struct(png_ptr);
    if (!end_info)
    {
        png_destroy_read_struct(&png_ptr, &info_ptr,
          (png_infopp)NULL);
        return;
    }

    if (setjmp(png_ptr->jmpbuf))
    {
        png_destroy_read_struct(&png_ptr, &info_ptr,
           &end_info);
        fclose(fp);
        return;
    }

	png_init_io(png_ptr, fp);

	png_set_sig_bytes(png_ptr, 8);

/* IHDR doesn't work */
	png_read_info(png_ptr, info_ptr);

    width = png_get_image_width(png_ptr, info_ptr);
    height = png_get_image_height(png_ptr, info_ptr);

	row_pointers = (png_bytep*) malloc(sizeof(png_bytep) * height);

	for (row = 0; row < height; row++)
	{
		row_pointers[row] = (png_bytep) malloc(png_get_rowbytes(png_ptr, info_ptr));
	}

	png_read_image(png_ptr, row_pointers);

	for (row = 0; row < ext_tile_height; row++) {
		for (i=0; i<ext_tile_width; i++) {
			for (j=0; j<4; j++) {
				res[row * ext_tile_width * 4 + i * 4 + j] = row_pointers[row][i*4+j];
			}
		}
	}
	
	return (void *) res;
}

void *ext_draw_image(int X, int Y, unsigned char *image) {
	int row, i, j;
	int temp;

         Uint32   pixel;
         Uint8   *bits, bpp;

	for (row=0; row<ext_tile_height; row++) {
		for (i=0; i<ext_tile_width; i++) {

			temp = row*ext_tile_width*4 + i*4;

			if (image[temp+3] > 127) {
		pixel = SDL_MapRGB(screen->format, image[temp], image[temp+1], image[temp+2]);
		 bpp = glob.bpp;
		bits = ((Uint8 *)screen->pixels)+(Y+row)*screen->pitch+(X+i)*bpp;

         /* Set the pixel */
         switch(bpp) {
                 case 1:
                         *((Uint8 *)(bits)) = (Uint8)pixel;
                         break;
                 case 2:
                         *((Uint16 *)(bits)) = (Uint16)pixel;
                         break;
                 case 3: { /* Format/endian independent */
                         Uint8 r, g, b;

                         r = (pixel>>screen->format->Rshift)&0xFF;
                         g = (pixel>>screen->format->Gshift)&0xFF;
                         b = (pixel>>screen->format->Bshift)&0xFF;
                         *((bits)+screen->format->Rshift/8) = r; 
                         *((bits)+screen->format->Gshift/8) = g;
                         *((bits)+screen->format->Bshift/8) = b;
                         }
                         break;
                 case 4:
                         *((Uint32 *)(bits)) = (Uint32)pixel;
                         break;
         }

			}

		}
	}
}
